// LICENSE : MIT
"use strict";

var _AcronymCreator = _interopRequireDefault(require("./AcronymCreator"));

var _wordUtils = require("./word-utils");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var isCapitalized = require("is-capitalized");

var includes = require("array-includes");

var defaultOptions = {
  min_acronym_len: 3,
  max_acronym_len: 5,
  ignore_acronyms: [],
  acronymJoiningWords: ["of", "the", "for", "in", "and", "or", "&"]
};
/*
Step

1. Collect Acronym => `acronymList`
2. Collect original words of Acronym => `expandedAcronymList`
3. expandedAcronymList.includes(acronymList)
    - Not found Acronym and throw error
 */

module.exports = function (context) {
  var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  var minAcronymLength = options.min_acronym_len || defaultOptions.min_acronym_len;
  var maxAcronymLength = options.max_acronym_len || defaultOptions.max_acronym_len;
  var ignoreAcronymList = options.ignore_acronyms || defaultOptions.ignore_acronyms;
  var acronymJoiningWords = options.acronymJoiningWords || defaultOptions.acronymJoiningWords;
  var {
    Syntax,
    RuleError,
    report,
    getSource
  } = context; // pickup acronyms

  var acronymList = []; // original words in document

  var expandedAcronymList = [];

  var isWordSatisfy = word => {
    if (word.length < minAcronymLength) {
      return false;
    } else if (word.length > maxAcronymLength) {
      return false;
    }

    return true;
  };

  return {
    [Syntax.Str](node) {
      var text = getSource(node);
      var words = text.split(/\b/);
      var acronymCreator = new _AcronymCreator.default();
      words.forEach(word => {
        if (word.trim().length === 0) {
          return;
        }

        if ((0, _wordUtils.isAllCapitals)(word)) {
          // collect Acronym
          var isOk = isWordSatisfy(word) && !includes(ignoreAcronymList, word);

          if (isOk && !includes(acronymList, word)) {
            acronymList.push(word);
          }
        } else if (isCapitalized(word)) {
          // Add temporarySequence
          // => add temp [Aword, Bword, Cword] = ABC
          acronymCreator.addWord(word);
        } else if (!includes(acronymJoiningWords, word) && // ignore of and...
        acronymCreator.canExtractAcronym()) {
          // Create Acronym
          var acronyms = acronymCreator.extractAcronym();
          acronyms.forEach(acronym => {
            if (isWordSatisfy(acronym)) {
              expandedAcronymList.push(acronym);
            }
          });
        }
      });

      if (acronymCreator.canExtractAcronym()) {
        // Create Acronym
        var acronyms = acronymCreator.extractAcronym();
        acronyms.forEach(acronym => {
          if (isWordSatisfy(acronym)) {
            expandedAcronymList.push(acronym);
          }
        });
      }
    },

    [Syntax.Document + ":exit"](node) {
      acronymList.forEach(acronym => {
        // not found acronym in document
        if (!includes(expandedAcronymList, acronym)) {
          report(node, new RuleError("\"".concat(acronym, "\" is unexpanded acronym. What does \"").concat(acronym, "\" stand for?")));
        }
      });
    }

  };
};
//# sourceMappingURL=textlint-rule-unexpanded-acronym.js.map