"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.isAllCapitals = isAllCapitals;
exports.expandOneWordToAcronym = expandOneWordToAcronym;
exports.expandWordsToAcronym = expandWordsToAcronym;

function isAllCapitals(word) {
  return /^[A-Z]+$/.test(word);
}
/**
 * Capital Word to Acronym
 * @param {string} CapitalWord
 * @returns {string} return Acronym
 */


function expandOneWordToAcronym(CapitalWord) {
  var acronym = CapitalWord.charAt(0);
  var restWord = CapitalWord.slice(1);
  var lastCapitalChar = null;

  for (var i = 0; i < restWord.length; i++) {
    var char = restWord.charAt(i);

    if (/[A-Z]/.test(char)) {
      lastCapitalChar = char;
    } else {
      if (lastCapitalChar) {
        acronym += lastCapitalChar;
        lastCapitalChar = null;
      }
    }
  }

  return acronym;
}
/*
 * create Acronym from words.
 * @param {string[]} words
 * @returns {string[]} string if only one word (2) array if multiple words
 */


function expandWordsToAcronym(words) {
  //XMLHttpRequest -> XHR
  if (words.length === 1) {
    return [expandOneWordToAcronym(words[0])];
  } else {
    var result = []; // In American Broadcast Company -> ["C", "BC", "ABC", "IABC"]

    var reversedWords = words.slice().reverse();
    reversedWords.reduce((acronym, word) => {
      acronym.unshift(word.charAt(0));
      result.push(acronym.join(""));
      return acronym;
    }, []); // In American Broadcast Company -> ["I", "IA", "IAB", "IABC"]

    words.reduce((acronym, word) => {
      acronym.push(word.charAt(0));
      result.push(acronym.join(""));
      return acronym;
    }, []);
    return result;
  }
}
//# sourceMappingURL=word-utils.js.map